@@ -1,29 +1,27 @@
 use std::fmt;
 
-use ruff_diagnostics::{Diagnostic, Fix};
-use ruff_diagnostics::{FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
-use ruff_python_ast::visitor;
+use ruff_macros::{ViolationMetadata, derive_message_formats};
+use ruff_python_ast::helpers::any_over_expr;
 use ruff_python_ast::visitor::Visitor;
-use ruff_python_ast::{self as ast, Arguments, Expr, ExprContext, Parameters, Stmt};
-use ruff_text_size::Ranged;
+use ruff_python_ast::{self as ast, Expr, ExprContext, Parameters, Stmt};
+use ruff_python_ast::{ExprLambda, visitor};
+use ruff_python_semantic::SemanticModel;
 
+use crate::Fix;
 use crate::checkers::ast::Checker;
-
 use crate::rules::flake8_comprehensions::fixes;
-
-use super::helpers;
+use crate::{FixAvailability, Violation};
 
 /// ## What it does
-/// Checks for unnecessary `map` calls with `lambda` functions.
+/// Checks for unnecessary `map()` calls with lambda functions.
 ///
 /// ## Why is this bad?
-/// Using `map(func, iterable)` when `func` is a `lambda` is slower than
+/// Using `map(func, iterable)` when `func` is a lambda is slower than
 /// using a generator expression or a comprehension, as the latter approach
 /// avoids the function call overhead, in addition to being more readable.
 ///
-/// This rule also applies to `map` calls within `list`, `set`, and `dict`
-/// calls. For example:
+/// This rule also applies to `map()` calls within `list()`, `set()`, and
+/// `dict()` calls. For example:
 ///
 /// - Instead of `list(map(lambda num: num * 2, nums))`, use
 ///   `[num * 2 for num in nums]`.
@@ -32,7 +30,7 @@ use super::helpers;
 /// - Instead of `dict(map(lambda v: (v, v ** 2), values))`, use
 ///   `{v: v ** 2 for v in values}`.
 ///
-/// ## Examples
+/// ## Example
 /// ```python
 /// map(lambda x: x + 1, iterable)
 /// ```
@@ -45,8 +43,9 @@ use super::helpers;
 /// ## Fix safety
 /// This rule's fix is marked as unsafe, as it may occasionally drop comments
 /// when rewriting the call. In most cases, though, comments will be preserved.
-#[violation]
-pub struct UnnecessaryMap {
+#[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.74")]
+pub(crate) struct UnnecessaryMap {
     object_type: ObjectType,
 }
 
@@ -56,165 +55,101 @@ impl Violation for UnnecessaryMap {
     #[derive_message_formats]
     fn message(&self) -> String {
         let UnnecessaryMap { object_type } = self;
-        format!("Unnecessary `map` usage (rewrite using a {object_type})")
+        format!("Unnecessary `map()` usage (rewrite using a {object_type})")
     }
 
     fn fix_title(&self) -> Option<String> {
         let UnnecessaryMap { object_type } = self;
-        Some(format!("Replace `map` with a {object_type}"))
+        Some(format!("Replace `map()` with a {object_type}"))
     }
 }
 
 /// C417
-pub(crate) fn unnecessary_map(
-    checker: &mut Checker,
-    expr: &Expr,
-    parent: Option<&Expr>,
-    func: &Expr,
-    args: &[Expr],
-) {
-    let Some(builtin_name) = checker.semantic().resolve_builtin_symbol(func) else {
+pub(crate) fn unnecessary_map(checker: &Checker, call: &ast::ExprCall) {
+    let semantic = checker.semantic();
+    let (func, arguments) = (&call.func, &call.arguments);
+
+    if !arguments.keywords.is_empty() {
+        return;
+    }
+
+    let Some(object_type) = ObjectType::from(func, semantic) else {
         return;
     };
 
-    let object_type = match builtin_name {
-        "map" => ObjectType::Generator,
-        "list" => ObjectType::List,
-        "set" => ObjectType::Set,
-        "dict" => ObjectType::Dict,
-        _ => return,
-    };
+    let parent = semantic.current_expression_parent();
 
-    match object_type {
+    let (lambda, iterables) = match object_type {
         ObjectType::Generator => {
+            let parent_call_func = match parent {
+                Some(Expr::Call(call)) => Some(&call.func),
+                _ => None,
+            };
+
             // Exclude the parent if already matched by other arms.
-            if parent
-                .and_then(Expr::as_call_expr)
-                .and_then(|call| call.func.as_name_expr())
-                .is_some_and(|name| matches!(name.id.as_str(), "list" | "set" | "dict"))
-            {
+            if parent_call_func.is_some_and(|func| is_list_set_or_dict(func, semantic)) {
                 return;
             }
 
-            // Only flag, e.g., `map(lambda x: x + 1, iterable)`.
-            let [Expr::Lambda(ast::ExprLambda {
-                parameters, body, ..
-            }), _] = args
-            else {
+            let Some(result) = map_lambda_and_iterables(call, semantic) else {
                 return;
             };
 
-            if parameters.as_ref().is_some_and(|parameters| {
-                late_binding(parameters, body)
-                    || parameters
-                        .iter_non_variadic_params()
-                        .any(|param| param.default.is_some())
-                    || parameters.vararg.is_some()
-                    || parameters.kwarg.is_some()
-            }) {
-                return;
-            }
+            result
         }
-        ObjectType::List | ObjectType::Set => {
-            // Only flag, e.g., `list(map(lambda x: x + 1, iterable))`.
-            let [Expr::Call(ast::ExprCall {
-                func,
-                arguments: Arguments { args, keywords, .. },
-                ..
-            })] = args
-            else {
+
+        ObjectType::List | ObjectType::Set | ObjectType::Dict => {
+            let [Expr::Call(inner_call)] = arguments.args.as_ref() else {
                 return;
             };
 
-            if args.len() != 2 {
-                return;
-            }
-
-            if !keywords.is_empty() {
-                return;
-            }
-
-            let Some(argument) = helpers::first_argument_with_matching_function("map", func, args)
-            else {
+            let Some((lambda, iterables)) = map_lambda_and_iterables(inner_call, semantic) else {
                 return;
             };
 
-            let Expr::Lambda(ast::ExprLambda {
-                parameters, body, ..
-            }) = argument
-            else {
-                return;
-            };
+            if object_type == ObjectType::Dict {
+                let (Expr::Tuple(ast::ExprTuple { elts, .. })
+                | Expr::List(ast::ExprList { elts, .. })) = &*lambda.body
+                else {
+                    return;
+                };
 
-            if parameters.as_ref().is_some_and(|parameters| {
-                late_binding(parameters, body)
-                    || parameters
-                        .iter_non_variadic_params()
-                        .any(|param| param.default.is_some())
-                    || parameters.vararg.is_some()
-                    || parameters.kwarg.is_some()
-            }) {
-                return;
-            }
-        }
-        ObjectType::Dict => {
-            // Only flag, e.g., `dict(map(lambda v: (v, v ** 2), values))`.
-            let [Expr::Call(ast::ExprCall {
-                func,
-                arguments: Arguments { args, keywords, .. },
-                ..
-            })] = args
-            else {
-                return;
-            };
-
-            if args.len() != 2 {
-                return;
+                if elts.len() != 2 {
+                    return;
+                }
             }
 
-            if !keywords.is_empty() {
-                return;
-            }
-
-            let Some(argument) = helpers::first_argument_with_matching_function("map", func, args)
-            else {
-                return;
-            };
-
-            let Expr::Lambda(ast::ExprLambda {
-                parameters, body, ..
-            }) = argument
-            else {
-                return;
-            };
-
-            let (Expr::Tuple(ast::ExprTuple { elts, .. }) | Expr::List(ast::ExprList { elts, .. })) =
-                body.as_ref()
-            else {
-                return;
-            };
-
-            if elts.len() != 2 {
-                return;
-            }
-
-            if parameters.as_ref().is_some_and(|parameters| {
-                late_binding(parameters, body)
-                    || parameters
-                        .iter_non_variadic_params()
-                        .any(|param| param.default.is_some())
-                    || parameters.vararg.is_some()
-                    || parameters.kwarg.is_some()
-            }) {
-                return;
-            }
+            (lambda, iterables)
         }
     };
 
-    let mut diagnostic = Diagnostic::new(UnnecessaryMap { object_type }, expr.range());
+    // If the lambda body contains a `yield` or `yield from`, rewriting `map(lambda ...)` to a
+    // generator expression or any comprehension is invalid Python syntax
+    // (e.g., `yield` is not allowed inside generator or comprehension expressions). In such cases, skip.
+    if lambda_contains_yield(&lambda.body) {
+        return;
+    }
+
+    for iterable in iterables {
+        // For example, (x+1 for x in (c:=a)) is invalid syntax
+        // so we can't suggest it.
+        if any_over_expr(iterable, &|expr| expr.is_named_expr()) {
+            return;
+        }
+
+        if iterable.is_starred_expr() {
+            return;
+        }
+    }
+
+    if !lambda_has_expected_arity(lambda) {
+        return;
+    }
+
+    let mut diagnostic = checker.report_diagnostic(UnnecessaryMap { object_type }, call.range);
     diagnostic.try_set_fix(|| {
         fixes::fix_unnecessary_map(
-            expr,
+            call,
             parent,
             object_type,
             checker.locator(),
@@ -222,7 +157,74 @@ pub(crate) fn unnecessary_map(
         )
         .map(Fix::unsafe_edit)
     });
-    checker.diagnostics.push(diagnostic);
+}
+
+fn is_list_set_or_dict(func: &Expr, semantic: &SemanticModel) -> bool {
+    semantic
+        .resolve_qualified_name(func)
+        .is_some_and(|qualified_name| {
+            matches!(
+                qualified_name.segments(),
+                ["" | "builtins", "list" | "set" | "dict"]
+            )
+        })
+}
+
+fn map_lambda_and_iterables<'a>(
+    call: &'a ast::ExprCall,
+    semantic: &'a SemanticModel,
+) -> Option<(&'a ExprLambda, &'a [Expr])> {
+    if !semantic.match_builtin_expr(&call.func, "map") {
+        return None;
+    }
+
+    let arguments = &call.arguments;
+
+    if !arguments.keywords.is_empty() {
+        return None;
+    }
+
+    let Some((Expr::Lambda(lambda), iterables)) = arguments.args.split_first() else {
+        return None;
+    };
+
+    Some((lambda, iterables))
+}
+
+/// Returns true if the expression tree contains a `yield` or `yield from` expression.
+fn lambda_contains_yield(expr: &Expr) -> bool {
+    any_over_expr(expr, &|expr| {
+        matches!(expr, Expr::Yield(_) | Expr::YieldFrom(_))
+    })
+}
+
+/// A lambda as the first argument to `map()` has the "expected" arity when:
+///
+/// * It has exactly one parameter
+/// * That parameter is not variadic
+/// * That parameter does not have a default value
+fn lambda_has_expected_arity(lambda: &ExprLambda) -> bool {
+    let Some(parameters) = lambda.parameters.as_deref() else {
+        return false;
+    };
+
+    let [parameter] = &*parameters.args else {
+        return false;
+    };
+
+    if parameter.default.is_some() {
+        return false;
+    }
+
+    if parameters.vararg.is_some() || parameters.kwarg.is_some() {
+        return false;
+    }
+
+    if late_binding(parameters, &lambda.body) {
+        return false;
+    }
+
+    true
 }
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
@@ -233,13 +235,25 @@ pub(crate) enum ObjectType {
     Dict,
 }
 
+impl ObjectType {
+    fn from(func: &Expr, semantic: &SemanticModel) -> Option<Self> {
+        match semantic.resolve_builtin_symbol(func) {
+            Some("map") => Some(Self::Generator),
+            Some("list") => Some(Self::List),
+            Some("set") => Some(Self::Set),
+            Some("dict") => Some(Self::Dict),
+            _ => None,
+        }
+    }
+}
+
 impl fmt::Display for ObjectType {
     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
         match self {
             ObjectType::Generator => fmt.write_str("generator expression"),
-            ObjectType::List => fmt.write_str("`list` comprehension"),
-            ObjectType::Set => fmt.write_str("`set` comprehension"),
-            ObjectType::Dict => fmt.write_str("`dict` comprehension"),
+            ObjectType::List => fmt.write_str("list comprehension"),
+            ObjectType::Set => fmt.write_str("set comprehension"),
+            ObjectType::Dict => fmt.write_str("dict comprehension"),
         }
     }
 }
