@@ -5,8 +5,8 @@ pub(crate) use bare_except::*;
 pub(crate) use blank_lines::*;
 pub(crate) use compound_statements::*;
 pub(crate) use doc_line_too_long::*;
+pub use errors::IOError;
 pub(crate) use errors::*;
-pub use errors::{IOError, SyntaxError};
 pub(crate) use invalid_escape_sequence::*;
 pub(crate) use lambda_assignment::*;
 pub(crate) use line_too_long::*;
@@ -17,8 +17,10 @@ pub(crate) use module_import_not_at_top_of_file::*;
 pub(crate) use multiple_imports_on_one_line::*;
 pub(crate) use not_tests::*;
 pub(crate) use tab_indentation::*;
+pub(crate) use too_many_newlines_at_end_of_file::*;
 pub(crate) use trailing_whitespace::*;
 pub(crate) use type_comparison::*;
+pub(crate) use whitespace_after_decorator::*;
 
 mod ambiguous_class_name;
 mod ambiguous_function_name;
@@ -39,5 +41,7 @@ mod module_import_not_at_top_of_file;
 mod multiple_imports_on_one_line;
 mod not_tests;
 mod tab_indentation;
+mod too_many_newlines_at_end_of_file;
 mod trailing_whitespace;
 mod type_comparison;
+mod whitespace_after_decorator;
